from models import EmailAddress
from django.forms import ModelForm, ValidationError
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _

class EmailAddressForm(ModelForm):
    def __init__(self, user=None, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        self.user = user

    class Meta:
        model = EmailAddress
        exclude = ('user', 'is_primary', 'is_active', 'identifier')

    def clean_email(self):
        """
        Ensure this email address is unique throughout the ``site`` identified by SITE_ID.
        """
        email = self.cleaned_data['email']
        try:
            EmailAddress.objects.get(user=self.user, email=email)
        except EmailAddress.DoesNotExist:
            try:
                User.objects.get(email=email)
            except User.DoesNotExist:
                return email
        
        raise ValidationError(u"This email address already in use.")

    def save(self):
        e = EmailAddress(**{'user': self.user, 'email': self.cleaned_data["email"]})
        return e.save()






from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from utils import get_unique_random
from django.db.models import signals

class EmailAddress(models.Model):

    user = models.ForeignKey(User, related_name="%(class)s")
    email = models.EmailField(_("Email Address"))
    is_primary = models.BooleanField(default=False)
    is_active = models.BooleanField(default=False)
    identifier = models.CharField(max_length=255, null=True)

    class Meta:
        verbose_name = _("email address")
        verbose_name_plural = _("email addresses")
        unique_together = (("user", "email"),)

    def __unicode__(self):
        return u"%s (%s)" % (self.email, self.user.username)

    def save(self, *args, **kwargs):
        if not self.identifier:
            self.identifier = get_unique_random(20).lower()
        super(EmailAddress, self).save(*args, **kwargs)


#########################################################
# stick signals listeners here to do some work for us
#########################################################
def create_primary_email_for_new_user(sender, instance, created, **kwargs):
    """
    Create a matching email address seperate from that of User object when a user object is created.
    """
    # user was just created, so no worries about duplicate emails as it has been done before
    if instance.email:
        try:
            EmailAddress.objects.get(user=instance, email__iexact=instance.email)
        except EmailAddress.DoesNotExist:
            e = EmailAddress(**{'user': instance, 
                                'email': instance.email, 
                                'is_primary': True,
                                'is_active': True})
            e.save()

def remove_all_emails_for_deleted_user(sender, instance, **kwargs):
    """
    Delete all emails addresses associated with this user that was just delete.
    """
    # user was just delete, delete any email associated with this user
    emails = EmailAddress.objects.filter(user=instance)
    for e in emails:
        e.delete()

# latch on signals here
signals.post_save.connect(create_primary_email_for_new_user, sender=User)
signals.post_delete.connect(remove_all_emails_for_deleted_user, sender=User)



import os
try:
    _s = os.environ['DJANGO_SETTINGS_MODULE']
except KeyError:
    # DJANGO_SETTINGS_MODULE should have been set by now, if not, we must be in test mode
    os.environ['DJANGO_SETTINGS_MODULE'] = 'emailmgr.tests.testsettings'




import os
from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='django-emailmgr',
    version='0.1',
    description = "A Django application that manages multiple email addresses for a given user",
    long_description = read('README'),
    author='Val L33',
    author_email='val@neekware.com',
    url='https://github.com/un33k/django-emailmgr',
    packages=['emailmgr'],
    #install_requires = [''],
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Web Environment',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Topic :: Utilities'],
    )




import os, random, string, time
from django.conf import settings
from django.db import models, IntegrityError
from django.template.loader import render_to_string
from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib.sites.models import Site
from django.contrib.auth.models import User
from django.utils.hashcompat import sha_constructor
from django.utils.translation import gettext_lazy as _
import defaults

# some people might like to user mailer by jtauber, accomodate them
if "mailer" in settings.INSTALLED_APPS:
    from mailer import send_mail
else:
    from django.core.mail import send_mail

# get a random string of known length
def get_unique_random(length=10):
    randtime = str(time.time()).split('.')[0]
    rand = ''.join([random.choice(randtime+string.letters+string.digits) for i in range(length)])
    return sha_constructor(rand).hexdigest()[:length]
 
# given a template name, return its path
def get_template(name):
    return os.path.join(getattr(defaults, "EMAIL_MGR_TEMPLATE_PATH"), name)


# send activation link to user's primary email address
def send_activation(request, identifier):
    # import pdb; pdb.set_trace()
    this_site = Site.objects.get_current()
    
    # first try to use our views fuction to construct the activation path (deterministic)
    # if views didn't reverse to a path, then use named url (less deterministic as it is user configurable)
    try:
        p = reverse("emailargs.view.email_activate",args=[identifier])
    except NoReverseMatch:
        p = reverse('emailmgr_email_send_activation', kwargs={'identifier': identifier})
            
    proto = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "")
    if not proto:
        if request.is_secure():
            proto = "https"
        else:
            proto = "http"

    url = u"%s://%s%s" % (proto, unicode(this_site.domain),p)
    context = {"user": request.user, "activate_url": url, "this_site": this_site,"identifier": identifier,}
    subject = render_to_string(get_template("email_activation_subject.txt"), context)
    subject = "".join(subject.splitlines())
    message = render_to_string(get_template("email_activation_message.txt"), context)
    send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, [request.user.email])








from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib import messages as Msg
from django.http import HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
from django.contrib.auth.models import User
from forms import EmailAddressForm
from models import EmailAddress
from utils import send_activation, get_template
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse


@login_required
def email_add(request):
    """
    User is logged and has a primary email address already
    This will add an aditional email address to this User
    """
    if request.method == 'POST':
        form = EmailAddressForm(user=request.user, data=request.POST)
        if form.is_valid():
            form.save()
            Msg.add_message (request, Msg.SUCCESS, _('email address added'))
            return HttpResponseRedirect(reverse('emailmgr_email_list'))
    else:
        form = EmailAddressForm(user=request.user)

    return render_to_response(get_template('emailmgr_email_add.html'),
                              {'add_email_form': form},
                              context_instance=RequestContext(request)
                              )

@login_required
def email_make_primary(request, identifier="somekey"):
    """
    User is logged in, has a second email that is already activated and 
    wants to make that the primary email address.
    The User objects' email address will also be replace with this newly
    primary email address, so Django internals work it the new primary address too
    """
    email = get_object_or_404(EmailAddress, identifier__iexact=identifier.lower())
    if email.is_active:
        if email.is_primary:
            Msg.add_message (request, Msg.SUCCESS, _('email address is already primary'))
        else:
            emails = EmailAddress.objects.filter(user=email.user)
            for e in emails:
                e.is_primary = False
                e.save()
    
            email.user.email = email.email
            email.user.save()
            email.is_primary = True
            email.save()
            Msg.add_message (request, Msg.SUCCESS, _('primary address changed'))
    else:
        Msg.add_message (request, Msg.SUCCESS, _('email address must be activated first'))

    return HttpResponseRedirect(reverse('emailmgr_email_list'))


@login_required
def email_send_activation(request, identifier="somekey"):
    """
    The user is logged in, has added a new email address to his/her account.
    User can do anything with the newly added email, unless it is first activated.
    This function will send an activation email to the currently primary email address 
    associated with the User's account
    """
    email = get_object_or_404(EmailAddress, identifier__iexact=identifier.lower())
    if email.is_active:
        Msg.add_message (request, Msg.SUCCESS, _('email address already activated'))
    else:
        send_activation(request, identifier)
        Msg.add_message (request, Msg.SUCCESS, _('activation email sent'))

    return HttpResponseRedirect(reverse('emailmgr_email_list'))


@login_required
def email_activate(request, identifier="somekey"):
    """
    User is already logged in and the activation link will trigger the email address
    in question to be activated. If the account is already active, then a message is 
    put in the message buffer indicating that the email is already active
    """
    email = get_object_or_404(EmailAddress, identifier__iexact=identifier.lower())
    if email.is_active:
        Msg.add_message (request, Msg.SUCCESS, _('email address already active'))
    else:
        email.is_active = True
        email.save()
        Msg.add_message (request, Msg.SUCCESS, _('email address is now active'))

    return HttpResponseRedirect(reverse('emailmgr_email_list'))

@login_required
def email_delete(request, identifier="somekey"):
    """
    Email needs to be removed from User's account, primary email address cannot be removed
    """
    email = get_object_or_404(EmailAddress, identifier__iexact=identifier.lower())
    if email.email == request.user.email:
        Msg.add_message (request, Msg.ERROR, _('cannot remove primary email address'))
    elif email.user != request.user:
        Msg.add_message (request, Msg.ERROR, _('email address is not associated with this account'))
    else:
        email.delete()
        Msg.add_message (request, Msg.SUCCESS, _('email address removed'))

    return HttpResponseRedirect(reverse('emailmgr_email_list'))


@login_required
def email_list(request):
    """
    All email address associated with User account will be passed into the template as a list
    An ``add`` email form will be passed in the template so user can add new email inline
    """
    email_add_form = EmailAddressForm(user=request.user)
    emails_list = EmailAddress.objects.all()
    return render_to_response(get_template('emailmgr_email_list.html'),
                              {
                                'emails_list': emails_list,
                                'add_email_form': email_add_form
                              },
                              context_instance=RequestContext(request)
                              )













# -*- coding: utf-8 -*-
from django.conf.urls.defaults import patterns, include, url
from django.conf import settings
from views import email_add, email_list, email_delete, \
            email_send_activation, email_activate, email_make_primary

#add an email to a User account
urlpatterns = patterns('',
    url(
        r'^email/add/$', 
        email_add, 
        name='emailmgr_email_add'
        ),
    url(
        r'^email/send_activation/(?P<identifier>\w+)/$',
        email_send_activation,
        name='emailmgr_email_send_activation'
        ),
    url(
        r'^email/activate/(?P<identifier>\w+)/$',
        email_activate,
        name='emailmgr_email_activate'
        ),
    url(
        r'^email/make_primary/(?P<identifier>\w+)/$',
        email_make_primary,
        name='emailmgr_email_make_primary'
        ),
    url(
        r'^email/delete/(?P<identifier>\w+)/$',
        email_delete,
        name='emailmgr_email_delete'
        ),
    url(
        r'^email/list/$',
        email_list,
        name='emailmgr_email_list'
        ),
)